@@ -8,8 +8,6 @@
  *
  * See the file "license.terms" for information on usage and redistribution of
  * this file, and for a DISCLAIMER OF ALL WARRANTIES.
- *
- * RCS: @(#) $Id: tkCanvUtil.c,v 1.18 2007/12/13 15:24:13 dgp Exp $
  */
 
 #include "tkInt.h"
@@ -268,12 +266,7 @@ Tk_CanvasGetCoordFromObj(
 {
     TkCanvas *canvasPtr = (TkCanvas *) canvas;
 
-    if (Tk_GetMMFromObj(canvasPtr->interp, canvasPtr->tkwin, obj,
-	    doublePtr) != TCL_OK) {
-	return TCL_ERROR;
-    }
-    *doublePtr *= canvasPtr->pixelsPerMM;
-    return TCL_OK;
+    return Tk_GetDoublePixelsFromObj(canvasPtr->interp, canvasPtr->tkwin, obj, doublePtr);
 }
 
 /*
@@ -897,7 +890,7 @@ Tk_GetDash(
 	goto badDashList;
     }
 
-    if (ABS(dash->number) > sizeof(char *)) {
+    if ((unsigned int)ABS(dash->number) > sizeof(char *)) {
 	ckfree((char *) dash->pattern.pt);
     }
     if (argc > (int)sizeof(char *)) {
@@ -938,7 +931,7 @@ Tk_GetDash(
     if (argv != NULL) {
 	ckfree((char *) argv);
     }
-    if (ABS(dash->number) > sizeof(char *)) {
+    if ((unsigned int)ABS(dash->number) > sizeof(char *)) {
 	ckfree((char *) dash->pattern.pt);
     }
     dash->number = 0;
@@ -1010,13 +1003,13 @@ Tk_DeleteOutline(
     if (outline->gc != None) {
 	Tk_FreeGC(display, outline->gc);
     }
-    if (ABS(outline->dash.number) > sizeof(char *)) {
+    if ((unsigned int)ABS(outline->dash.number) > sizeof(char *)) {
 	ckfree((char *) outline->dash.pattern.pt);
     }
-    if (ABS(outline->activeDash.number) > sizeof(char *)) {
+    if ((unsigned int)ABS(outline->activeDash.number) > sizeof(char *)) {
 	ckfree((char *) outline->activeDash.pattern.pt);
     }
-    if (ABS(outline->disabledDash.number) > sizeof(char *)) {
+    if ((unsigned int)ABS(outline->disabledDash.number) > sizeof(char *)) {
 	ckfree((char *) outline->disabledDash.pattern.pt);
     }
     if (outline->color != NULL) {
@@ -1141,12 +1134,10 @@ Tk_ConfigOutlineGC(
     if (mask && (dash->number != 0)) {
 	gcValues->line_style = LineOnOffDash;
 	gcValues->dash_offset = outline->offset;
-	if (dash->number >= 2) {
-	    gcValues->dashes = 4;
-	} else if (dash->number > 0) {
+	if (dash->number > 0) {
 	    gcValues->dashes = dash->pattern.array[0];
 	} else {
-	    gcValues->dashes = (char) (4 * width);
+	    gcValues->dashes = (char) (4 * width + 0.5);
 	}
 	mask |= GCLineStyle|GCDashList|GCDashOffset;
     }
@@ -1226,7 +1217,7 @@ Tk_ChangeOutlineGC(
     }
 
     if ((dash->number<-1) ||
-	    ((dash->number == -1) && (dash->pattern.array[1] != ','))) {
+	    ((dash->number == -1) && (dash->pattern.array[0] != ','))) {
 	char *q;
 	int i = -dash->number;
 
@@ -1346,13 +1337,11 @@ Tk_ResetOutlineGC(
 
     if ((dash->number > 2) || (dash->number < -1) || (dash->number==2 &&
 	    (dash->pattern.array[0] != dash->pattern.array[1])) ||
-	    ((dash->number == -1) && (dash->pattern.array[1] != ','))) {
-	if (dash->number < 0) {
-	    dashList = (int) (4 * width + 0.5);
-	} else if (dash->number<3) {
+	    ((dash->number == -1) && (dash->pattern.array[0] != ','))) {
+	if (dash->number > 0) {
 	    dashList = dash->pattern.array[0];
 	} else {
-	    dashList = 4;
+	    dashList = (char) (4 * width + 0.5);
 	}
 	XSetDashes(((TkCanvas *)canvas)->display, outline->gc,
 		outline->offset, &dashList , 1);
@@ -1445,7 +1434,7 @@ Tk_CanvasPsOutline(
 	str = (char *)ckalloc((unsigned int) (1 - 8*dash->number));
 	lptr = (char *)ckalloc((unsigned int) (1 - 2*dash->number));
     }
-    ptr = (ABS(dash->number) > sizeof(char *)) ?
+    ptr = ((unsigned int)ABS(dash->number) > sizeof(char *)) ?
 	    dash->pattern.pt : dash->pattern.array;
     if (dash->number > 0) {
 	char *ptr0 = ptr;
@@ -1671,8 +1660,10 @@ TkCanvTranslatePath(
     double *a, *b, *t;		/* Pointers to parts of the temporary
 				 * storage */
     int i, j;			/* Loop counters */
+#ifndef NDEBUG
     int maxOutput;		/* Maximum number of outputs that we will
 				 * allow */
+#endif
     double limit[4];		/* Boundries at which clipping occurs */
     double staticSpace[480];	/* Temp space from the stack */
 
@@ -1767,7 +1758,9 @@ TkCanvTranslatePath(
      * This is the loop that makes the four passes through the data.
      */
 
+#ifndef NDEBUG
     maxOutput = numVertex*3;
+#endif
     for (j=0; j<4; j++){
 	double xClip = limit[j];
 	int inside = a[0]<xClip;
