@@ -10,8 +10,6 @@
  *
  * See the file "license.terms" for information on usage and redistribution of
  * this file, and for a DISCLAIMER OF ALL WARRANTIES.
- *
- * RCS: @(#) $Id: tkColor.c,v 1.16 2007/12/13 15:24:13 dgp Exp $
  */
 
 #include "tkInt.h"
@@ -129,8 +127,8 @@ Tk_AllocColorFromObj(
      */
 
     if (tkColPtr != NULL) {
-	TkColor *firstColorPtr = (TkColor *)
-		Tcl_GetHashValue(tkColPtr->hashPtr);
+	TkColor *firstColorPtr = Tcl_GetHashValue(tkColPtr->hashPtr);
+
 	FreeColorObjProc(objPtr);
 	for (tkColPtr = firstColorPtr; tkColPtr != NULL;
 		tkColPtr = tkColPtr->nextPtr) {
@@ -204,7 +202,7 @@ Tk_GetColor(
 
     nameHashPtr = Tcl_CreateHashEntry(&dispPtr->colorNameTable, name, &isNew);
     if (!isNew) {
-	existingColPtr = (TkColor *) Tcl_GetHashValue(nameHashPtr);
+	existingColPtr = Tcl_GetHashValue(nameHashPtr);
 	for (tkColPtr = existingColPtr; tkColPtr != NULL;
 		tkColPtr = tkColPtr->nextPtr) {
 	    if ((tkColPtr->screen == Tk_Screen(tkwin))
@@ -300,8 +298,11 @@ Tk_GetColorByValue(
 
     /*
      * First, check to see if there's already a mapping for this color name.
+     * Must clear the structure first; it's not tightly packed on 64-bit
+     * systems. [Bug 2911570]
      */
 
+    memset(&valueKey, 0, sizeof(ValueKey));
     valueKey.red = colorPtr->red;
     valueKey.green = colorPtr->green;
     valueKey.blue = colorPtr->blue;
@@ -310,7 +311,7 @@ Tk_GetColorByValue(
     valueHashPtr = Tcl_CreateHashEntry(&dispPtr->colorValueTable,
 	    (char *) &valueKey, &isNew);
     if (!isNew) {
-	tkColPtr = (TkColor *) Tcl_GetHashValue(valueHashPtr);
+	tkColPtr = Tcl_GetHashValue(valueHashPtr);
 	tkColPtr->resourceRefCount++;
 	return &tkColPtr->color;
     }
@@ -415,8 +416,8 @@ Tk_GCForColor(
 
     if (tkColPtr->gc == None) {
 	gcValues.foreground = tkColPtr->color.pixel;
-	tkColPtr->gc = XCreateGC(DisplayOfScreen(tkColPtr->screen),
-		drawable, GCForeground, &gcValues);
+	tkColPtr->gc = XCreateGC(DisplayOfScreen(tkColPtr->screen), drawable,
+		GCForeground, &gcValues);
     }
     return tkColPtr->gc;
 }
@@ -474,7 +475,7 @@ Tk_FreeColor(
     }
     TkpFreeColor(tkColPtr);
 
-    prevPtr = (TkColor *) Tcl_GetHashValue(tkColPtr->hashPtr);
+    prevPtr = Tcl_GetHashValue(tkColPtr->hashPtr);
     if (prevPtr == tkColPtr) {
 	if (tkColPtr->nextPtr == NULL) {
 	    Tcl_DeleteHashEntry(tkColPtr->hashPtr);
@@ -664,7 +665,7 @@ Tk_GetColorFromObj(
     if (hashPtr == NULL) {
 	goto error;
     }
-    for (tkColPtr = (TkColor *) Tcl_GetHashValue(hashPtr);
+    for (tkColPtr = Tcl_GetHashValue(hashPtr);
 	    (tkColPtr != NULL); tkColPtr = tkColPtr->nextPtr) {
 	if ((Tk_Screen(tkwin) == tkColPtr->screen)
 		&& (Tk_Colormap(tkwin) == tkColPtr->colormap)) {
@@ -780,7 +781,7 @@ TkDebugColor(
     resultPtr = Tcl_NewObj();
     hashPtr = Tcl_FindHashEntry(&dispPtr->colorNameTable, name);
     if (hashPtr != NULL) {
-	TkColor *tkColPtr = (TkColor *) Tcl_GetHashValue(hashPtr);
+	TkColor *tkColPtr = Tcl_GetHashValue(hashPtr);
 
 	if (tkColPtr == NULL) {
 	    Tcl_Panic("TkDebugColor found empty hash table entry");
@@ -797,7 +798,110 @@ TkDebugColor(
     }
     return resultPtr;
 }
-
+
+#ifndef __WIN32__
+
+/* This function is not necessary for Win32,
+ * since XParseColor already does the right thing */
+
+#undef XParseColor
+
+CONST char *CONST tkWebColors[20] = {
+    /* 'a' */ "qua\0#0000ffffffff",
+    /* 'b' */ NULL,
+    /* 'c' */ "rimson\0#dcdc14143c3c",
+    /* 'd' */ NULL,
+    /* 'e' */ NULL,
+    /* 'f' */ "uchsia\0#ffff0000ffff",
+    /* 'g' */ NULL,
+    /* 'h' */ NULL,
+    /* 'i' */ "ndigo\0#4b4b00008282",
+    /* 'j' */ NULL,
+    /* 'k' */ NULL,
+    /* 'l' */ "ime\0#0000ffff0000",
+    /* 'm' */ NULL,
+    /* 'n' */ NULL,
+    /* 'o' */ "live\0#808080800000",
+    /* 'p' */ NULL,
+    /* 'q' */ NULL,
+    /* 'r' */ NULL,
+    /* 's' */ "ilver\0#c0c0c0c0c0c0",
+    /* 't' */ "eal\0#000080808080"
+};
+
+Status
+TkParseColor(
+    Display *display,		/* The display */
+    Colormap map,			/* Color map */
+    const char *name,     /* String to be parsed */
+    XColor *color)
+{
+    char buf[14];
+    if (*name == '#') {
+	buf[0] = '#'; buf[13] = '\0';
+	if (!*(++name) || !*(++name) || !*(++name)) {
+	    /* Not at least 3 hex digits, so invalid */
+	return 0;
+	} else if (!*(++name)) {
+	    /* Exactly 3 hex digits */
+	    buf[9] = buf[10] = buf[11] = buf[12] = *(--name);
+	    buf[5] = buf[6] = buf[7] = buf[8] = *(--name);
+	    buf[1] = buf[2] = buf[3] = buf[4] = *(--name);
+	    name = buf;
+	} else if (!*(++name)	|| !*(++name)) {
+	    /* Not at least 6 hex digits, so invalid */
+	    return 0;
+	} else if (!*(++name)) {
+	    /* Exactly 6 hex digits */
+	    buf[10] = buf[12] = *(--name);
+	    buf[9] = buf[11] = *(--name);
+	    buf[6] = buf[8] = *(--name);
+	    buf[5] = buf[7] = *(--name);
+	    buf[2] = buf[4] = *(--name);
+	    buf[1] = buf[3] = *(--name);
+	    name = buf;
+	} else if (!*(++name) || !*(++name)) {
+	    /* Not at least 9 hex digits, so invalid */
+	    return 0;
+	} else if (!*(++name)) {
+	    /* Exactly 9 hex digits */
+	    buf[11] = *(--name);
+	    buf[10] = *(--name);
+	    buf[9] = buf[12] = *(--name);
+	    buf[7] = *(--name);
+	    buf[6] = *(--name);
+	    buf[5] = buf[8] = *(--name);
+	    buf[3] = *(--name);
+	    buf[2] = *(--name);
+	    buf[1] = buf[4] = *(--name);
+	    name = buf;
+	} else if (!*(++name) || !*(++name) || *(++name)) {
+	    /* Not exactly 12 hex digits, so invalid */
+	    return 0;
+	} else {
+	    name -= 13;
+	}
+	goto done;
+    } else if (((*name - 'A') & 0xdf) < sizeof(tkWebColors)/sizeof(tkWebColors[0])) {
+	const char *p = tkWebColors[((*name - 'A') & 0x1f)];
+	if (p) {
+	    const char *q = name;
+	    while (!((*p - *(++q)) & 0xdf)) {
+		if (!*p++) {
+		    name = p;
+		    goto done;
+		}
+	    }
+	}
+    }
+    if (strlen(name) > 99) {
+	/* Don't bother to parse this. [Bug 2809525]*/
+	return 0;
+    }
+done:
+    return XParseColor(display, map, name, color);
+}
+#endif /* __WIN32__ */
 /*
  * Local Variables:
  * mode: c
