@@ -413,8 +413,8 @@ typedef struct
 static Tk_OptionSpec CheckbuttonOptionSpecs[] =
 {
     {TK_OPTION_STRING, "-variable", "variable", "Variable",
-	"", Tk_Offset(Checkbutton, checkbutton.variableObj), -1,
-	TK_OPTION_DONT_SET_DEFAULT,0,0},
+	NULL, Tk_Offset(Checkbutton, checkbutton.variableObj), -1,
+	TK_OPTION_NULL_OK,0,0},
     {TK_OPTION_STRING, "-onvalue", "onValue", "OnValue",
 	"1", Tk_Offset(Checkbutton, checkbutton.onValueObj), -1,
 	0,0,0},
