@@ -10,7 +10,7 @@
 
 #include <string.h>
 #include <stdio.h>
-#include <tk.h>
+#include <tkInt.h>
 #include <X11/Xatom.h>
 
 #include "ttkTheme.h"
@@ -155,8 +155,6 @@ typedef struct {
 #define DEF_LIST_HEIGHT	"10"
 
 static Tk_OptionSpec EntryOptionSpecs[] = {
-    WIDGET_TAKES_FOCUS,
-
     {TK_OPTION_BOOLEAN, "-exportselection", "exportSelection",
         "ExportSelection", "1", -1, Tk_Offset(Entry, entry.exportSelection),
 	0,0,0 },
@@ -200,6 +198,7 @@ static Tk_OptionSpec EntryOptionSpecs[] = {
 	NULL, Tk_Offset(Entry, entry.styleData.backgroundObj), -1,
 	TK_OPTION_NULL_OK,0,0},
 
+    WIDGET_TAKEFOCUS_TRUE,
     WIDGET_INHERIT_OPTIONS(ttkCoreOptionSpecs)
 };
 
@@ -1136,13 +1135,14 @@ EntryDoLayout(void *recordPtr)
  *      Get a GC using the specified foreground color and the entry's font.
  *      Result must be freed with Tk_FreeGC().
  */
-static GC EntryGetGC(Entry *entryPtr, Tcl_Obj *colorObj)
+static GC EntryGetGC(Entry *entryPtr, Tcl_Obj *colorObj, TkRegion clip)
 {
     Tk_Window tkwin = entryPtr->core.tkwin;
     Tk_Font font = Tk_GetFontFromObj(tkwin, entryPtr->entry.fontObj);
     XColor *colorPtr;
     unsigned long mask = 0ul;
     XGCValues gcValues;
+    GC gc;
 
     gcValues.line_width = 1; mask |= GCLineWidth;
     gcValues.font = Tk_FontId(font); mask |= GCFont;
@@ -1150,7 +1150,11 @@ static GC EntryGetGC(Entry *entryPtr, Tcl_Obj *colorObj)
 	gcValues.foreground = colorPtr->pixel;
 	mask |= GCForeground;
     }
-    return Tk_GetGC(entryPtr->core.tkwin, mask, &gcValues);
+    gc = Tk_GetGC(entryPtr->core.tkwin, mask, &gcValues);
+    if (clip != None) {
+	TkSetRegion(Tk_Display(entryPtr->core.tkwin), gc, clip);
+    }
+    return gc;
 }
 
 /* EntryDisplay --
@@ -1161,15 +1165,19 @@ static void EntryDisplay(void *clientData, Drawable d)
     Entry *entryPtr = clientData;
     Tk_Window tkwin = entryPtr->core.tkwin;
     int leftIndex = entryPtr->entry.xscroll.first,
-	rightIndex = entryPtr->entry.xscroll.last,
+	rightIndex = entryPtr->entry.xscroll.last + 1,
 	selFirst = entryPtr->entry.selectFirst,
 	selLast = entryPtr->entry.selectLast;
     EntryStyleData es;
     GC gc;
     int showSelection, showCursor;
+    Ttk_Box textarea;
+    TkRegion clipRegion;
+    XRectangle rect;
 
     EntryInitStyleData(entryPtr, &es);
 
+    textarea = Ttk_ClientRegion(entryPtr->core.layout, "textarea");
     showCursor =
 	   (entryPtr->core.flags & CURSOR_ON) != 0
 	&& EntryEditable(entryPtr)
@@ -1215,6 +1223,20 @@ static void EntryDisplay(void *clientData, Drawable d)
 	}
     }
 
+    /* Initialize the clip region. Note that Xft does _not_ derive its
+     * clipping area from the GC, so we have to supply that by other means.
+     */
+
+    rect.x = textarea.x;
+    rect.y = textarea.y;
+    rect.width = textarea.width;
+    rect.height = textarea.height;
+    clipRegion = TkCreateRegion();
+    TkUnionRectWithRegion(&rect, clipRegion, clipRegion);
+#ifdef HAVE_XFT
+    TkUnixSetXftClipRegion(clipRegion);
+#endif
+
     /* Draw cursor:
      */
     if (showCursor) {
@@ -1231,31 +1253,42 @@ static void EntryDisplay(void *clientData, Drawable d)
 	/* @@@ should: maybe: SetCaretPos even when blinked off */
 	Tk_SetCaretPos(tkwin, cursorX, cursorY, cursorHeight);
 
-	gc = EntryGetGC(entryPtr, es.insertColorObj);
+	gc = EntryGetGC(entryPtr, es.insertColorObj, clipRegion);
 	XFillRectangle(Tk_Display(tkwin), d, gc,
 	    cursorX-cursorWidth/2, cursorY, cursorWidth, cursorHeight);
+	XSetClipMask(Tk_Display(tkwin), gc, None);
 	Tk_FreeGC(Tk_Display(tkwin), gc);
     }
 
     /* Draw the text:
      */
-    gc = EntryGetGC(entryPtr, es.foregroundObj);
+    gc = EntryGetGC(entryPtr, es.foregroundObj, clipRegion);
     Tk_DrawTextLayout(
 	Tk_Display(tkwin), d, gc, entryPtr->entry.textLayout,
 	entryPtr->entry.layoutX, entryPtr->entry.layoutY,
 	leftIndex, rightIndex);
+    XSetClipMask(Tk_Display(tkwin), gc, None);
     Tk_FreeGC(Tk_Display(tkwin), gc);
 
     /* Overwrite the selected portion (if any) in the -selectforeground color:
      */
     if (showSelection) {
-	gc = EntryGetGC(entryPtr, es.selForegroundObj);
+	gc = EntryGetGC(entryPtr, es.selForegroundObj, clipRegion);
 	Tk_DrawTextLayout(
 	    Tk_Display(tkwin), d, gc, entryPtr->entry.textLayout,
 	    entryPtr->entry.layoutX, entryPtr->entry.layoutY,
 	    selFirst, selLast);
+	XSetClipMask(Tk_Display(tkwin), gc, None);
 	Tk_FreeGC(Tk_Display(tkwin), gc);
     }
+
+    /* Drop the region. Note that we have to manually remove the reference to
+     * it from the Xft guts (if they're being used).
+     */
+#ifdef HAVE_XFT
+    TkUnixSetXftClipRegion(None);
+#endif
+    TkDestroyRegion(clipRegion);
 }
 
 /*------------------------------------------------------------------------
@@ -1609,6 +1642,14 @@ static int EntryXViewCommand(
     void *recordPtr, Tcl_Interp *interp, int objc, Tcl_Obj *const objv[])
 {
     Entry *entryPtr = recordPtr;
+    if (objc == 3) {
+	int newFirst;
+	if (EntryIndex(interp, entryPtr, objv[2], &newFirst) != TCL_OK) {
+	    return TCL_ERROR;
+	}
+	TtkScrollTo(entryPtr->entry.xscrollHandle, newFirst);
+	return TCL_OK;
+    }
     return TtkScrollviewCommand(interp, objc, objv, entryPtr->entry.xscrollHandle);
 }
 
