@@ -45,8 +45,6 @@ typedef struct
 
 static Tk_OptionSpec ScaleOptionSpecs[] =
 {
-    WIDGET_TAKES_FOCUS,
-
     {TK_OPTION_STRING, "-command", "command", "Command", "",
 	Tk_Offset(Scale,scale.commandObj), -1, 
 	TK_OPTION_NULL_OK,0,0},
@@ -68,6 +66,7 @@ static Tk_OptionSpec ScaleOptionSpecs[] =
 	DEF_SCALE_LENGTH, Tk_Offset(Scale,scale.lengthObj), -1, 0, 0, 
     	GEOMETRY_CHANGED},
 
+    WIDGET_TAKEFOCUS_TRUE,
     WIDGET_INHERIT_OPTIONS(ttkCoreOptionSpecs)
 };
 
