@@ -1,5 +1,4 @@
-/* $Id: ttkScroll.c,v 1.5 2006/12/18 19:33:13 jenglish Exp $
- *
+/*
  * Copyright 2004, Joe English
  *
  * Support routines for scrollable widgets.
@@ -35,7 +34,7 @@
  * TtkScrollbarUpdateRequired, which will invoke step (5) (@@@ Fix this)
  */
 
-#include <tk.h>
+#include <tkInt.h>
 #include "ttkTheme.h"
 #include "ttkWidget.h"
 
@@ -76,8 +75,10 @@ static int UpdateScrollbar(Tcl_Interp *interp, ScrollHandle h)
 {
     Scrollable *s = h->scrollPtr;
     WidgetCore *corePtr = h->corePtr;
-    char args[TCL_DOUBLE_SPACE * 2];
+    char arg1[TCL_DOUBLE_SPACE + 2];
+    char arg2[TCL_DOUBLE_SPACE + 2];
     int code;
+    Tcl_DString buf;
 
     h->flags &= ~SCROLL_UPDATE_REQUIRED;
 
@@ -85,12 +86,17 @@ static int UpdateScrollbar(Tcl_Interp *interp, ScrollHandle h)
 	return TCL_OK;
     }
 
-    sprintf(args, " %g %g",
-		(double)s->first / s->total,
-		(double)s->last / s->total);
+    arg1[0] = arg2[0] = ' ';
+    Tcl_PrintDouble(interp, (double)s->first / s->total, arg1+1);
+    Tcl_PrintDouble(interp, (double)s->last / s->total, arg2+1);
+    Tcl_DStringInit(&buf);
+    Tcl_DStringAppend(&buf, s->scrollCmd, -1);
+    Tcl_DStringAppend(&buf, arg1, -1);
+    Tcl_DStringAppend(&buf, arg2, -1);
 
     Tcl_Preserve(corePtr);
-    code = Tcl_VarEval(interp, s->scrollCmd, args, NULL);
+    code = Tcl_EvalEx(interp, Tcl_DStringValue(&buf), -1, TCL_EVAL_GLOBAL);
+    Tcl_DStringFree(&buf);
     if (WidgetDestroyed(corePtr)) {
 	Tcl_Release(corePtr);
 	return TCL_ERROR;
@@ -190,11 +196,10 @@ int TtkScrollviewCommand(
     int newFirst = s->first;
 
     if (objc == 2) {
-	char buf[TCL_DOUBLE_SPACE * 2];
-	sprintf(buf, "%g %g",
-		(double)s->first / s->total,
-		(double)s->last / s->total);
-	Tcl_SetResult(interp, buf, TCL_VOLATILE);
+	Tcl_Obj *result[2];
+	result[0] = Tcl_NewDoubleObj((double)s->first / s->total);
+	result[1] = Tcl_NewDoubleObj((double)s->last / s->total);
+	Tcl_SetObjResult(interp, Tcl_NewListObj(2, result));
 	return TCL_OK;
     } else if (objc == 3) {
 	if (Tcl_GetIntFromObj(interp, objv[2], &newFirst) != TCL_OK) {
