@@ -16,9 +16,9 @@
 #ifndef _TKMACDEFAULT
 #define _TKMACDEFAULT
 
-#ifndef TK_MAC_BUTTON_USE_COMPATIBILITY_METRICS
-#define TK_MAC_BUTTON_USE_COMPATIBILITY_METRICS 1
-#endif
+//#ifndef TK_MAC_BUTTON_USE_COMPATIBILITY_METRICS
+//#define TK_MAC_BUTTON_USE_COMPATIBILITY_METRICS 1
+//#endif
 
 /*
  * The definitions below provide symbolic names for the default colors.
@@ -72,12 +72,12 @@
 #define DEF_BUTTON_HIGHLIGHT_BG_MONO	DEF_BUTTON_BG_MONO
 #define DEF_BUTTON_HIGHLIGHT		"systemButtonFrame"
 #define DEF_LABEL_HIGHLIGHT_WIDTH	"0"
-#if TK_MAC_BUTTON_USE_COMPATIBILITY_METRICS
-#define DEF_BUTTON_HIGHLIGHT_WIDTH	"4"
-#define DEF_BUTTON_HIGHLIGHT_WIDTH_NOCM	"1"
-#else
+//#if TK_MAC_BUTTON_USE_COMPATIBILITY_METRICS
+//#define DEF_BUTTON_HIGHLIGHT_WIDTH	"4"
+//#define DEF_BUTTON_HIGHLIGHT_WIDTH_NOCM	"1"
+//#else
 #define DEF_BUTTON_HIGHLIGHT_WIDTH	"1"
-#endif
+//#endif
 #define DEF_BUTTON_IMAGE		((char *) NULL)
 #define DEF_BUTTON_INDICATOR		"1"
 #define DEF_BUTTON_JUSTIFY		"center"
@@ -85,19 +85,19 @@
 #define DEF_BUTTON_ON_VALUE		"1"
 #define DEF_BUTTON_TRISTATE_VALUE	""
 #define DEF_BUTTON_OVER_RELIEF		""
-#if TK_MAC_BUTTON_USE_COMPATIBILITY_METRICS
-#define DEF_BUTTON_PADX			"12"
-#define DEF_BUTTON_PADX_NOCM		"1"
-#else
+//#if TK_MAC_BUTTON_USE_COMPATIBILITY_METRICS
+//#define DEF_BUTTON_PADX			"12"
+//#define DEF_BUTTON_PADX_NOCM		"1"
+//#else
 #define DEF_BUTTON_PADX			"1"
-#endif
+//#endif
 #define DEF_LABCHKRAD_PADX		"1"
-#if TK_MAC_BUTTON_USE_COMPATIBILITY_METRICS
-#define DEF_BUTTON_PADY			"3"
-#define DEF_BUTTON_PADY_NOCM		"1"
-#else
+//#if TK_MAC_BUTTON_USE_COMPATIBILITY_METRICS
+//#define DEF_BUTTON_PADY			"3"
+//#define DEF_BUTTON_PADY_NOCM		"1"
+//#else
 #define DEF_BUTTON_PADY			"1"
-#endif
+//#endif
 #define DEF_LABCHKRAD_PADY		"1"
 #define DEF_BUTTON_RELIEF		"flat"
 #define DEF_LABCHKRAD_RELIEF		"flat"
@@ -409,6 +409,7 @@
 #define DEF_PANEDWINDOW_HEIGHT		""
 #define DEF_PANEDWINDOW_OPAQUERESIZE	"1"
 #define DEF_PANEDWINDOW_ORIENT		"horizontal"
+#define DEF_PANEDWINDOW_PROXYBORDER	"2"
 #define DEF_PANEDWINDOW_RELIEF		"flat"
 #define DEF_PANEDWINDOW_SASHCURSOR	""
 #define DEF_PANEDWINDOW_SASHPAD		"0"
