@@ -30,7 +30,6 @@ static char tkLibPath[PATH_MAX + 1] = "";
 
 static char scriptPath[PATH_MAX + 1] = "";
 
-int tkMacOSXGCEnabled = 0;
 long tkMacOSXMacOSXVersion = 0;
 
 #pragma mark TKApplication(TKInit) 
@@ -60,14 +59,24 @@ static void keyboardChanged(CFNotificationCenterRef center, void *observer, CFSt
 @end
 
 @implementation TKApplication
+@synthesize poolProtected = _poolProtected;
 @end
 
 @implementation TKApplication(TKInit)
+- (void) _resetAutoreleasePool
+{
+    if(![self poolProtected]) {
+	[_mainPool drain];
+	_mainPool = [NSAutoreleasePool new];
+    }
+}
+
 #ifdef TK_MAC_DEBUG_NOTIFICATIONS
 - (void)_postedNotification:(NSNotification *)notification {
     TKLog(@"-[%@(%p) %s] %@", [self class], self, _cmd, notification);
 }
 #endif
+
 - (void)_setupApplicationNotifications {
     NSNotificationCenter *nc = [NSNotificationCenter defaultCenter];
 #define observe(n, s) [nc addObserver:self selector:@selector(s) name:(n) object:nil]
@@ -82,14 +91,18 @@ static void keyboardChanged(CFNotificationCenterRef center, void *observer, CFSt
     CFNotificationCenterAddObserver(CFNotificationCenterGetDistributedCenter(), NULL, &keyboardChanged, kTISNotifySelectedKeyboardInputSourceChanged, NULL, CFNotificationSuspensionBehaviorCoalesce);
 #endif
 }
-- (void)_setupEventLoop {
 
-    /*Remove private API calls here.*/
+- (void)_setupEventLoop {
+    NSAutoreleasePool *pool = [NSAutoreleasePool new];
     [self finishLaunching];
     [self setWindowsNeedUpdate:YES];
+    [pool drain];
 }
+
 - (void)_setup:(Tcl_Interp *)interp {
     _eventInterp = interp;
+    _mainPool = [NSAutoreleasePool new];
+    [NSApp setPoolProtected:NO];
     _defaultMainMenu = nil;
     [self _setupMenus];
     [self setDelegate:self];
@@ -100,8 +113,10 @@ static void keyboardChanged(CFNotificationCenterRef center, void *observer, CFSt
     [self _setupWindowNotifications];
     [self _setupApplicationNotifications];
 }
+
 - (NSString *)tkFrameworkImagePath:(NSString*)image {
     NSString *path = nil;
+    NSAutoreleasePool *pool = [NSAutoreleasePool new];
     if (tkLibPath[0] != '\0') {
 	path = [[NSBundle bundleWithPath:[[NSString stringWithUTF8String:
 		tkLibPath] stringByAppendingString:@"/../.."]]
@@ -132,6 +147,8 @@ static void keyboardChanged(CFNotificationCenterRef center, void *observer, CFSt
 	}
     }
 #endif
+    [path retain];
+    [pool drain];
     return path;
 }
 @end
@@ -158,6 +175,7 @@ static void
 SetApplicationIcon(
     ClientData clientData)
 {
+    NSAutoreleasePool *pool = [NSAutoreleasePool new];
     NSString *path = [NSApp tkFrameworkImagePath:@"Tk.icns"];
     if (path) {
 	NSImage *image = [[NSImage alloc] initWithContentsOfFile:path];
@@ -166,6 +184,7 @@ SetApplicationIcon(
 	    [image release];
 	}
     }
+    [pool drain];
 }
 
 /*
@@ -243,20 +262,19 @@ TkpInit(
 	}
 #endif
 
-	static NSAutoreleasePool *pool = nil;
-	if (!pool) {
-	    pool = [NSAutoreleasePool new];
+	{
+	    NSAutoreleasePool *pool = [NSAutoreleasePool new];
+		[[NSUserDefaults standardUserDefaults] registerDefaults:
+		     [NSDictionary dictionaryWithObjectsAndKeys:
+		     [NSNumber numberWithBool:YES],
+		     @"_NSCanWrapButtonTitles",
+		     [NSNumber numberWithInt:-1],
+		     @"NSStringDrawingTypesetterBehavior",
+		     nil]];
+	    [TKApplication sharedApplication];
+	    [pool drain];
+	    [NSApp _setup:interp];
 	}
-	tkMacOSXGCEnabled = ([NSGarbageCollector defaultCollector] != nil);
-	[[NSUserDefaults standardUserDefaults] registerDefaults:
-		[NSDictionary dictionaryWithObjectsAndKeys:
-		[NSNumber numberWithBool:YES],
-		@"_NSCanWrapButtonTitles",
-		[NSNumber numberWithInt:-1],
-		@"NSStringDrawingTypesetterBehavior",
-		nil]];
-	[TKApplication sharedApplication];
-	[NSApp _setup:interp];
 
 	/* Check whether we are a bundled executable: */
 	bundleRef = CFBundleGetMainBundle();
@@ -313,13 +331,15 @@ TkpInit(
 	    Tcl_DoWhenIdle(SetApplicationIcon, NULL);
 	}
 
-	[NSApp _setupEventLoop];
-	TkMacOSXInitAppleEvents(interp);
-	TkMacOSXUseAntialiasedText(interp, -1);
-	TkMacOSXInitCGDrawing(interp, TRUE, 0);
-	[pool drain];
-	pool = [NSAutoreleasePool new];
-
+	{
+	    NSAutoreleasePool *pool = [NSAutoreleasePool new];
+	    [NSApp _setupEventLoop];
+	    TkMacOSXInitAppleEvents(interp);
+	    TkMacOSXUseAntialiasedText(interp, -1);
+	    TkMacOSXInitCGDrawing(interp, TRUE, 0);
+	    [pool drain];
+	}
+	
 	/*
 	 * FIXME: Close stdin & stdout for remote debugging otherwise we will
 	 * fight with gdb for stdin & stdout
@@ -475,9 +495,8 @@ TkpDisplayWarning(
 MODULE_SCOPE void
 TkMacOSXDefaultStartupScript(void)
 {
-    CFBundleRef bundleRef;
-
-    bundleRef = CFBundleGetMainBundle();
+    NSAutoreleasePool *pool = [NSAutoreleasePool new];
+    CFBundleRef bundleRef = CFBundleGetMainBundle();
 
     if (bundleRef != NULL) {
 	CFURLRef appMainURL = CFBundleCopyResourceURL(bundleRef,
@@ -501,6 +520,7 @@ TkMacOSXDefaultStartupScript(void)
 	    CFRelease(appMainURL);
 	}
     }
+    [pool drain];
 }
 
 /*
