@@ -9,7 +9,7 @@
  * See the file "license.terms" for information on usage and redistribution
  * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
  */
-
+ 
 #ifndef _TKMACPRIV
 #define _TKMACPRIV
 
@@ -141,23 +141,6 @@
 		STRINGIFY(symbol)); \
     }
 
-/*
- * Macros for GC
- */
-
-#define TkMacOSXMakeUncollectable(x) ({ id o = (id)(x); \
-    if (o) { if(tkMacOSXGCEnabled) CFRetain(o); } o; })
-#define TkMacOSXMakeUncollectableAndRetain(x) ({ id o = (id)(x); \
-    if (o) { if(tkMacOSXGCEnabled) CFRetain(o); else [o retain]; } o; })
-#define TkMacOSXMakeCollectable(x) ({ id o = (id)(x); \
-    if (o) { x = nil; if (tkMacOSXGCEnabled) CFRelease(o); } o; })
-#define TkMacOSXMakeCollectableAndRelease(x) ({ id o = (id)(x); \
-    if (o) { x = nil; if (tkMacOSXGCEnabled) CFRelease(o); \
-    else [o release]; } o; })
-#define TkMacOSXMakeCollectableAndAutorelease(x) ({ id o = (id)(x); \
-    if (o) {  x = nil; if (tkMacOSXGCEnabled) CFRelease(o); \
-    else [o autorelease]; } o; })
-
 /*
  * Structure encapsulating current drawing environment.
  */
@@ -176,11 +159,7 @@ typedef struct TkMacOSXDrawingContext {
 
 MODULE_SCOPE CGFloat tkMacOSXZeroScreenHeight;
 MODULE_SCOPE CGFloat tkMacOSXZeroScreenTop;
-MODULE_SCOPE int tkMacOSXGCEnabled;
 MODULE_SCOPE long tkMacOSXMacOSXVersion;
-#if TK_MAC_BUTTON_USE_COMPATIBILITY_METRICS
-MODULE_SCOPE int tkMacOSXUseCompatibilityMetrics;
-#endif
 
 /*
  * Prototypes for TkMacOSXRegion.c.
@@ -293,10 +272,17 @@ VISIBILITY_HIDDEN
     TKMenu *_defaultMainMenu, *_defaultApplicationMenu;
     NSArray *_defaultApplicationMenuItems, *_defaultWindowsMenuItems;
     NSArray *_defaultHelpMenuItems;
+    NSWindow *_windowWithMouse;
+    NSAutoreleasePool *_mainPool;
+#ifdef __i386__
+    BOOL _poolProtected;
+#endif
 }
+@property BOOL poolProtected;
 @end
 @interface TKApplication(TKInit)
 - (NSString *)tkFrameworkImagePath:(NSString*)image;
+- (void)_resetAutoreleasePool;
 @end
 @interface TKApplication(TKEvent)
 - (NSEvent *)tkProcessEvent:(NSEvent *)theEvent;
@@ -314,15 +300,33 @@ VISIBILITY_HIDDEN
 - (void)tkProvidePasteboard:(TkDisplay *)dispPtr;
 - (void)tkCheckPasteboard;
 @end
+@interface TKApplication(TKHLEvents)
+- (void) terminate: (id) sender;
+- (void) preferences: (id) sender;
+- (void) handleQuitApplicationEvent:   (NSAppleEventDescriptor *)event 
+		     withReplyEvent:   (NSAppleEventDescriptor *)replyEvent;
+- (void) handleOpenApplicationEvent:   (NSAppleEventDescriptor *)event 
+		     withReplyEvent:   (NSAppleEventDescriptor *)replyEvent;
+- (void) handleReopenApplicationEvent: (NSAppleEventDescriptor *)event 
+		       withReplyEvent: (NSAppleEventDescriptor *)replyEvent;
+- (void) handleShowPreferencesEvent:   (NSAppleEventDescriptor *)event
+		     withReplyEvent:   (NSAppleEventDescriptor *)replyEvent;
+- (void) handleOpenDocumentsEvent:     (NSAppleEventDescriptor *)event 
+		   withReplyEvent:     (NSAppleEventDescriptor *)replyEvent;
+- (void) handlePrintDocumentsEvent:    (NSAppleEventDescriptor *)event 
+		   withReplyEvent:     (NSAppleEventDescriptor *)replyEvent;
+- (void) handleDoScriptEvent:          (NSAppleEventDescriptor *)event 
+		   withReplyEvent:     (NSAppleEventDescriptor *)replyEvent;
+@end
 
 VISIBILITY_HIDDEN
 @interface TKContentView : NSView <NSTextInput> {
 @private
   /*Remove private API calls.*/
-   #if 0
+#if 0
     id _savedSubviews;
     BOOL _subviewsSetAside;
-    #endif
+#endif
     NSString *privateWorkingText;
 }
 @end
@@ -331,10 +335,27 @@ VISIBILITY_HIDDEN
 - (void) deleteWorkingText;
 @end
 
+@interface TKContentView(TKWindowEvent)
+- (void) drawRect: (NSRect) rect;
+- (void) generateExposeEvents: (HIShapeRef) shape;
+- (void) generateExposeEvents: (HIShapeRef) shape childrenOnly: (int) childrenOnly;
+- (void) viewDidEndLiveResize;
+- (void) tkToolbarButton: (id) sender;
+- (BOOL) isOpaque;
+- (BOOL) wantsDefaultClipping;
+- (BOOL) acceptsFirstResponder;
+- (void) keyDown: (NSEvent *) theEvent;
+@end
+
 VISIBILITY_HIDDEN
 @interface TKWindow : NSWindow
 @end
 
+@interface NSWindow(TKWm)
+- (NSPoint) convertPointToScreen:(NSPoint)point;
+- (NSPoint) convertPointFromScreen:(NSPoint)point;
+@end
+
 #pragma mark NSMenu & NSMenuItem Utilities
 
 @interface NSMenu(TKUtils)
@@ -363,5 +384,4 @@ VISIBILITY_HIDDEN
 	keyEquivalentModifierMask:(NSUInteger)keyEquivalentModifierMask;
 @end
 
-
 #endif /* _TKMACPRIV */
