@@ -6,6 +6,8 @@
  *
  * Copyright 2001-2009, Apple Inc.
  * Copyright (c) 2005-2009 Daniel A. Steffen <das@users.sourceforge.net>
+ * Copyright (c) 2015 Kevin Walzer/WordTech Communications LLC.
+ * Copyright (c) 2015 Marc Culler.
  *
  * See the file "license.terms" for information on usage and redistribution of
  * this file, and for a DISCLAIMER OF ALL WARRANTIES.
@@ -15,7 +17,7 @@
 #include "tkMacOSXWm.h"
 #include "tkMacOSXEvent.h"
 #include "tkMacOSXDebug.h"
-
+ 
 /*
 #ifdef TK_MAC_DEBUG
 #define TK_MAC_DEBUG_EVENTS
@@ -27,7 +29,7 @@
  * Declaration of functions used only in this file
  */
 
-static int		GenerateUpdates(HIMutableShapeRef updateRgn,
+static int		GenerateUpdates(HIShapeRef updateRgn,
 			    CGRect *updateBounds, TkWindow *winPtr);
 static int		GenerateActivateEvents(TkWindow *winPtr,
 			    int activeFlag);
@@ -46,7 +48,7 @@ extern NSString *NSWindowDidOrderOffScreenNotification;
 #endif
 #endif
 
-extern NSString *opaqueTag;
+extern BOOL opaqueTag;
 
 @implementation TKApplication(TKWindowEvent)
 
@@ -163,6 +165,10 @@ extern NSString *opaqueTag;
 
     if (winPtr) {
 	TkGenWMDestroyEvent((Tk_Window) winPtr);
+	if (_windowWithMouse == w) {
+	    _windowWithMouse = nil;
+	    [w release];
+	}
     }
 
     /*
@@ -314,7 +320,7 @@ extern NSString *opaqueTag;
 
 static int
 GenerateUpdates(
-    HIMutableShapeRef updateRgn,
+    HIShapeRef updateRgn,
     CGRect *updateBounds,
     TkWindow *winPtr)
 {
@@ -745,15 +751,16 @@ TkWmProtocolEventProc(
 int
 Tk_MacOSXIsAppInFront(void)
 {
-    OSStatus err;
-    ProcessSerialNumber frontPsn, ourPsn = {0, kCurrentProcess};
     Boolean isFrontProcess = true;
+#if MAC_OS_X_VERSION_MIN_REQUIRED < 1060
+    ProcessSerialNumber frontPsn, ourPsn = {0, kCurrentProcess};
 
-    err = ChkErr(GetFrontProcess, &frontPsn);
-    if (err == noErr) {
-	ChkErr(SameProcess, &frontPsn, &ourPsn, &isFrontProcess);
+    if (noErr == GetFrontProcess(&frontPsn)){
+	SameProcess(&frontPsn, &ourPsn, &isFrontProcess);
     }
-
+#else
+    isFrontProcess = [NSRunningApplication currentApplication].active;
+#endif
     return (isFrontProcess == true);
 }
 
@@ -762,36 +769,24 @@ Tk_MacOSXIsAppInFront(void)
 #import <ApplicationServices/ApplicationServices.h>
 
 /*
- * Custom content view for Tk NSWindows, containing standard NSView subviews.
- * The goal is to emulate X11-style drawing in response to Expose events:
- * during the normal AppKit drawing cycle, we supress drawing of all subviews
- * and instead send Expose events about the subviews that would be redrawn.
+ * Custom content view for use in Tk NSWindows.
+ * 
+ * Since Tk handles all drawing of widgets, we only use the AppKit event loop
+ * as a source of input events.  To do this, we overload the NSView drawRect
+ * method with a method which generates Expose events for Tk but does no
+ * drawing.  The redrawing operations are then done when Tk processes these
+ * events.
+ *
+ * Earlier versions of Mac Tk used subclasses of NSView, e.g. NSButton, as the
+ * basis for Tk widgets.  These would then appear as subviews of the
+ * TKContentView.  To prevent the AppKit from redrawing and corrupting the Tk
+ * Widgets it was necessary to use Apple private API calls.  In order to avoid
+ * using private API calls, the NSView-based widgets have been replaced with
+ * normal Tk widgets which draw themselves as native widgets by using the
+ * HITheme API.
+ *
  */
 
-@interface TKContentView(TKWindowEvent)
-- (void) drawRect: (NSRect) rect;
-- (void) generateExposeEvents: (HIMutableShapeRef) shape;
-- (void) viewDidEndLiveResize;
-- (void) viewWillDraw;
-- (BOOL) isOpaque;
-- (BOOL) wantsDefaultClipping;
-- (BOOL) acceptsFirstResponder;
-- (void) keyDown: (NSEvent *) theEvent;
-@end
-
-@implementation TKContentView
-@end
-
-double drawTime;
-
-/*
- * Set a minimum time for drawing to render. With removal of private NSView API's, default drawing
- * is slower and less responsive. This number, which seems feasible after some experimentatation, skips
- * some drawing to avoid lag. 
- */
-
-#define MAX_DYNAMIC_TIME .000000001
-
 /*Restrict event processing to Expose events.*/
 static Tk_RestrictAction
 ExposeRestrictProc(
@@ -802,6 +797,14 @@ ExposeRestrictProc(
 	    ? TK_PROCESS_EVENT : TK_DEFER_EVENT);
 }
 
+/*Restrict event processing to ConfigureNotify events.*/
+static Tk_RestrictAction
+ConfigureRestrictProc(
+    ClientData arg,
+    XEvent *eventPtr)
+{
+    return (eventPtr->type==ConfigureNotify ? TK_PROCESS_EVENT : TK_DEFER_EVENT);
+}
 
 @implementation TKContentView(TKWindowEvent)
 
@@ -809,8 +812,9 @@ ExposeRestrictProc(
 {
     const NSRect *rectsBeingDrawn;
     NSInteger rectsBeingDrawnCount;
-
+    
     [self getRectsBeingDrawn:&rectsBeingDrawn count:&rectsBeingDrawnCount];
+
 #ifdef TK_MAC_DEBUG_DRAWING
     TKLog(@"-[%@(%p) %s%@]", [self class], self, _cmd, NSStringFromRect(rect));
     [[NSColor colorWithDeviceRed:0.0 green:1.0 blue:0.0 alpha:.1] setFill];
@@ -818,88 +822,140 @@ ExposeRestrictProc(
 	    NSCompositeSourceOver);
 #endif
 
-    NSDate *beginTime=[NSDate date];
-
-    /*Skip drawing during live resize if redraw is too slow.*/
-    if([self inLiveResize] && drawTime>MAX_DYNAMIC_TIME) return;
-
+ 	    
     CGFloat height = [self bounds].size.height;
     HIMutableShapeRef drawShape = HIShapeCreateMutable();
 
     while (rectsBeingDrawnCount--) {
 	CGRect r = NSRectToCGRect(*rectsBeingDrawn++);
-
 	r.origin.y = height - (r.origin.y + r.size.height);
 	HIShapeUnionWithRect(drawShape, &r);
     }
     if (CFRunLoopGetMain() == CFRunLoopGetCurrent()) {
-	[self generateExposeEvents:drawShape];
+	[self generateExposeEvents:(HIShapeRef)drawShape];
     } else {
 	[self performSelectorOnMainThread:@selector(generateExposeEvents:)
 		withObject:(id)drawShape waitUntilDone:NO
 		modes:[NSArray arrayWithObjects:NSRunLoopCommonModes,
+
 			NSEventTrackingRunLoopMode, NSModalPanelRunLoopMode,
 			nil]];
     }
+   
     CFRelease(drawShape);
-    drawTime=-[beginTime timeIntervalSinceNow];
+  
 }
 
-/*At conclusion of resize event, send notification and set view for redraw if earlier drawing was skipped because of lagginess.*/
-- (void)viewDidEndLiveResize
+-(void) setFrameSize: (NSSize)newsize
 {
-    if(drawTime>MAX_DYNAMIC_TIME) {
-    [self setNeedsDisplay:YES];
-    [super viewDidEndLiveResize];
+    [super setFrameSize: newsize];
+    if ([self inLiveResize]) {
+	NSWindow *w = [self window];
+	TkWindow *winPtr = TkMacOSXGetTkWindow(w);
+	Tk_Window tkwin = (Tk_Window) winPtr;
+	unsigned int width = (unsigned int)newsize.width;
+	unsigned int height=(unsigned int)newsize.height;
+	ClientData oldArg;
+    	Tk_RestrictProc *oldProc;
+
+	/* This can be called from outside the Tk event loop.
+	 * Since it calls Tcl_DoOneEvent, we need to make sure we
+	 * don't clobber the AutoreleasePool set up by the caller.
+	 */
+	[NSApp setPoolProtected:YES];
+	
+	/*
+	 * Try to prevent flickers and flashes.
+	 */
+	[w disableFlushWindow];
+	NSDisableScreenUpdates();
+	
+	/* Disable Tk drawing until the window has been completely configured.*/
+	TkMacOSXSetDrawingEnabled(winPtr, 0);
+
+	 /* Generate and handle a ConfigureNotify event for the new size.*/
+	TkGenWMConfigureEvent(tkwin, Tk_X(tkwin), Tk_Y(tkwin), width, height,
+			      TK_SIZE_CHANGED | TK_MACOSX_HANDLE_EVENT_IMMEDIATELY);
+    	oldProc = Tk_RestrictEvents(ConfigureRestrictProc, NULL, &oldArg);
+	while (Tk_DoOneEvent(TK_X_EVENTS|TK_DONT_WAIT)) {}
+    	Tk_RestrictEvents(oldProc, oldArg, &oldArg);
+
+	/* Now that Tk has configured all subwindows we can create the clip regions. */
+	TkMacOSXSetDrawingEnabled(winPtr, 1);
+	TkMacOSXInvalClipRgns(tkwin);
+	TkMacOSXUpdateClipRgn(winPtr);
+
+	 /* Finally, generate and process expose events to redraw the window. */
+	HIRect bounds = NSRectToCGRect([self bounds]);
+	HIShapeRef shape = HIShapeCreateWithRect(&bounds);
+	[self generateExposeEvents: shape];
+	while (Tk_DoOneEvent(TK_ALL_EVENTS|TK_DONT_WAIT)) {}
+	[w enableFlushWindow];
+	[w flushWindowIfNeeded];
+	NSEnableScreenUpdates();
+	[NSApp setPoolProtected:NO];
     }
 }
 
--(void) viewWillDraw  {
-	[self setNeedsDisplay:YES];
-    } 
+/*
+ * As insurance against bugs that might cause layout glitches during a live
+ * resize, we redraw the window one more time at the end of the resize
+ * operation.
+ */
 
-- (void) generateExposeEvents: (HIMutableShapeRef) shape
+- (void)viewDidEndLiveResize
 {
+    HIRect bounds = NSRectToCGRect([self bounds]);
+    HIShapeRef shape = HIShapeCreateWithRect(&bounds);
+    [super viewDidEndLiveResize];
+    [self generateExposeEvents: shape];
 
+}
+
+/* Core method of this class: generates expose events for redrawing.
+ * Whereas drawRect is intended to be called only from the Appkit event
+ * loop, this can be called from Tk.  If the Tcl_ServiceMode is set to
+ * TCL_SERVICE_ALL then the expose events will be immediately removed
+ * from the Tcl event loop and processed.  Typically, they should be queued,
+ * however.
+ */
+- (void) generateExposeEvents: (HIShapeRef) shape
+{
+    [self generateExposeEvents:shape childrenOnly:0];
+}
+
+- (void) generateExposeEvents: (HIShapeRef) shape
+		 childrenOnly: (int) childrenOnly
+{
     TkWindow *winPtr = TkMacOSXGetTkWindow([self window]);
     unsigned long serial;
     CGRect updateBounds;
+    int updatesNeeded;
 
     if (!winPtr) {
 		return;
     }
 
+    /* Generate Tk Expose events. */
     HIShapeGetBounds(shape, &updateBounds);
+    /* All of these events will share the same serial number. */
     serial = LastKnownRequestProcessed(Tk_Display(winPtr));
-    if (GenerateUpdates(shape, &updateBounds, winPtr) &&
-	![[NSRunLoop currentRunLoop] currentMode] &&
-	Tcl_GetServiceMode() != TCL_SERVICE_NONE) {
-    	/*
-    	 * Ensure there are no pending idle-time redraws that could prevent the
-    	 * just posted Expose events from generating new redraws.
-    	 */
+    updatesNeeded = GenerateUpdates(shape, &updateBounds, winPtr);
 
-    	while (Tcl_DoOneEvent(TCL_IDLE_EVENTS|TCL_DONT_WAIT)) {}
-
-    	/*
-    	 * For smoother drawing, process Expose events and resulting redraws
-    	 * immediately instead of at idle time.
-    	 */
-
-    	ClientData oldArg;
+    /* Process the Expose events if the service mode is TCL_SERVICE_ALL */
+    if (updatesNeeded && Tcl_GetServiceMode() == TCL_SERVICE_ALL) {
+	ClientData oldArg;
     	Tk_RestrictProc *oldProc = Tk_RestrictEvents(ExposeRestrictProc,
 						     UINT2PTR(serial), &oldArg);
-
     	while (Tcl_ServiceEvent(TCL_WINDOW_EVENTS)) {}
- 
     	Tk_RestrictEvents(oldProc, oldArg, &oldArg);
-    	while (Tcl_DoOneEvent(TCL_IDLE_EVENTS|TCL_DONT_WAIT)) {}
-
-    } 
-   
+    }
 }
 
-/*This is no-op on 10.7 and up because Apple has removed this widget, but leaving here for backwards compatibility.*/
+/*
+ * This is no-op on 10.7 and up because Apple has removed this widget,
+ * but we are leaving it here for backwards compatibility.
+ */
 - (void) tkToolbarButton: (id) sender
 {
 #ifdef TK_MAC_DEBUG_EVENTS
@@ -909,7 +965,6 @@ ExposeRestrictProc(
     int x, y;
     TkWindow *winPtr = TkMacOSXGetTkWindow([self window]);
     Tk_Window tkwin = (Tk_Window) winPtr;
-
     bzero(&event, sizeof(XVirtualEvent));
     event.type = VirtualEvent;
     event.serial = LastKnownRequestProcessed(Tk_Display(tkwin));
@@ -927,21 +982,11 @@ ExposeRestrictProc(
     Tk_QueueWindowEvent((XEvent *) &event, TCL_QUEUE_TAIL);
 }
 
-- (void) setFrameSize: (NSSize) newSize
-{
-    [super setFrameSize:newSize];
-}
-
-- (void) setNeedsDisplayInRect: (NSRect) invalidRect
-{
-    [super setNeedsDisplayInRect:invalidRect];
-}
-
 - (BOOL) isOpaque
 {
     NSWindow *w = [self window];
 
-    if (opaqueTag != NULL) {
+    if (opaqueTag) {
       return YES;
 	} else {
 
@@ -968,7 +1013,6 @@ ExposeRestrictProc(
 }
 
 @end
-
 
 /*
  * Local Variables:
