@@ -8,12 +8,25 @@
  *
  * See the file "license.terms" for information on usage and redistribution of
  * this file, and for a DISCLAIMER OF ALL WARRANTIES.
- *
- * RCS: @(#) $Id: tkUnixKey.c,v 1.12 2007/02/27 14:52:57 dkf Exp $
  */
 
 #include "tkInt.h"
 
+/*
+** Bug [3607830]: Before using Xkb, it must be initialized.  TkpOpenDisplay
+**                does this and sets the USE_XKB flag if xkb is supported.
+**                (should this be function ptr?)
+*/
+#ifdef HAVE_XKBKEYCODETOKEYSYM
+#  include <X11/XKBlib.h>
+#else
+#  define XkbKeycodeToKeysym(D,K,G,L) XKeycodeToKeysym(D,K,L)
+#endif
+#define TkKeycodeToKeysym(D,K,G,L)		\
+    ((D)->flags & TK_DISPLAY_USE_XKB) ?		\
+      XkbKeycodeToKeysym((D)->display,K,G,L) :	\
+      XKeycodeToKeysym((D)->display,K,L)
+
 /*
  * Prototypes for local functions defined in this file:
  */
@@ -24,16 +37,9 @@
  * Tk_SetCaretPos --
  *
  *	This enables correct placement of the XIM caret. This is called by
- *	widgets to indicate their cursor placement, and the caret location is
- *	used by TkpGetString to place the XIM caret. This is currently only
+ *	widgets to indicate their cursor placement.  This is currently only
  *	used for over-the-spot XIM.
  *
- * Results:
- *	None
- *
- * Side effects:
- *	None
- *
  *----------------------------------------------------------------------
  */
 
@@ -44,16 +50,42 @@ Tk_SetCaretPos(
     int y,
     int height)
 {
-    TkCaret *caretPtr = &(((TkWindow *) tkwin)->dispPtr->caret);
+    TkWindow *winPtr = (TkWindow *) tkwin;
+    TkDisplay *dispPtr = winPtr->dispPtr;
 
+    if (   dispPtr->caret.winPtr == winPtr
+	&& dispPtr->caret.x == x
+	&& dispPtr->caret.y == y
+	&& dispPtr->caret.height == height)
+    {
+	return;
+    }
+
+    dispPtr->caret.winPtr = winPtr;
+    dispPtr->caret.x = x;
+    dispPtr->caret.y = y;
+    dispPtr->caret.height = height;
+
+#ifdef TK_USE_INPUT_METHODS
     /*
-     * Use height for best placement of the XIM over-the-spot box.
+     * Adjust the XIM caret position.
      */
+    if (   (dispPtr->flags & TK_DISPLAY_USE_IM)
+	&& (dispPtr->inputStyle & XIMPreeditPosition)
+	&& (winPtr->inputContext != NULL) )
+    {
+	XVaNestedList preedit_attr;
+	XPoint spot;
 
-    caretPtr->winPtr = ((TkWindow *) tkwin);
-    caretPtr->x = x;
-    caretPtr->y = y;
-    caretPtr->height = height;
+	spot.x = dispPtr->caret.x;
+	spot.y = dispPtr->caret.y + dispPtr->caret.height;
+	preedit_attr = XVaCreateNestedList(0, XNSpotLocation, &spot, NULL);
+	XSetICValues(winPtr->inputContext,
+		XNPreeditAttributes, preedit_attr,
+		NULL);
+	XFree(preedit_attr);
+    }
+#endif
 }
 
 /*
@@ -76,38 +108,60 @@ Tk_SetCaretPos(
 
 char *
 TkpGetString(
-    TkWindow *winPtr,		/* Window where event occurred: needed to get
-				 * input context. */
+    TkWindow *winPtr,		/* Window where event occurred */
     XEvent *eventPtr,		/* X keyboard event. */
-    Tcl_DString *dsPtr)		/* Uninitialized or empty string to hold
-				 * result. */
+    Tcl_DString *dsPtr)		/* Initialized, empty string to hold result. */
 {
     int len;
     Tcl_DString buf;
-    Status status;
-#ifdef TK_USE_INPUT_METHODS
-    TkDisplay *dispPtr = winPtr->dispPtr;
-#endif
+    TkKeyEvent *kePtr = (TkKeyEvent *) eventPtr;
 
     /*
-     * Overallocate the dstring to the maximum stack amount.
+     * If we have the value cached already, use it now. [Bug 1373712]
      */
 
-    Tcl_DStringInit(&buf);
-    Tcl_DStringSetLength(&buf, TCL_DSTRING_STATIC_SIZE-1);
+    if (kePtr->charValuePtr != NULL) {
+	Tcl_DStringSetLength(dsPtr, kePtr->charValueLen);
+	memcpy(Tcl_DStringValue(dsPtr), kePtr->charValuePtr,
+		(unsigned) kePtr->charValueLen+1);
+	return Tcl_DStringValue(dsPtr);
+    }
 
 #ifdef TK_USE_INPUT_METHODS
-    if ((dispPtr->flags & TK_DISPLAY_USE_IM)
+    if ((winPtr->dispPtr->flags & TK_DISPLAY_USE_IM)
 	    && (winPtr->inputContext != NULL)
-	    && (eventPtr->type == KeyPress)) {
-#if TK_XIM_SPOT
-	XVaNestedList preedit_attr;
-	XPoint spot;
-#endif
+	    && (eventPtr->type == KeyPress))
+    {
+	Status status;
+
+#if X_HAVE_UTF8_STRING
+	Tcl_DStringSetLength(dsPtr, TCL_DSTRING_STATIC_SIZE-1);
+	len = Xutf8LookupString(winPtr->inputContext, &eventPtr->xkey,
+		Tcl_DStringValue(dsPtr), Tcl_DStringLength(dsPtr),
+		&kePtr->keysym, &status);
+
+	if (status == XBufferOverflow) { /* Expand buffer and try again */
+	    Tcl_DStringSetLength(dsPtr, len);
+	    len = Xutf8LookupString(winPtr->inputContext, &eventPtr->xkey,
+		    Tcl_DStringValue(dsPtr), Tcl_DStringLength(dsPtr),
+		    &kePtr->keysym, &status);
+	}
+	if ((status != XLookupChars) && (status != XLookupBoth)) {
+	    len = 0;
+	}
+	Tcl_DStringSetLength(dsPtr, len);
+#else /* !X_HAVE_UTF8_STRING */
+	/*
+	 * Overallocate the dstring to the maximum stack amount.
+	 */
+
+	Tcl_DStringInit(&buf);
+	Tcl_DStringSetLength(&buf, TCL_DSTRING_STATIC_SIZE-1);
 
 	len = XmbLookupString(winPtr->inputContext, &eventPtr->xkey,
-		Tcl_DStringValue(&buf), Tcl_DStringLength(&buf), NULL,
-		&status);
+		Tcl_DStringValue(&buf), Tcl_DStringLength(&buf),
+                &kePtr->keysym, &status);
+
 	/*
 	 * If the buffer wasn't big enough, grow the buffer and try again.
 	 */
@@ -115,47 +169,65 @@ TkpGetString(
 	if (status == XBufferOverflow) {
 	    Tcl_DStringSetLength(&buf, len);
 	    len = XmbLookupString(winPtr->inputContext, &eventPtr->xkey,
-		    Tcl_DStringValue(&buf), len, NULL, &status);
+		    Tcl_DStringValue(&buf), len, &kePtr->keysym, &status);
 	}
 	if ((status != XLookupChars) && (status != XLookupBoth)) {
 	    len = 0;
 	}
 
-#if TK_XIM_SPOT
+	Tcl_DStringSetLength(&buf, len);
+	Tcl_ExternalToUtfDString(NULL, Tcl_DStringValue(&buf), len, dsPtr);
+	Tcl_DStringFree(&buf);
+#endif /* X_HAVE_UTF8_STRING */
+    } else
+#endif /* TK_USE_INPUT_METHODS */
+    {
 	/*
-	 * Adjust the XIM caret position. We might want to check that this is
-	 * the right caret.winPtr as well.
+	 * Fall back to convert a keyboard event to a UTF-8 string using
+	 * XLookupString. This is used when input methods are turned off and
+	 * for KeyRelease events.
+	 *
+	 * Note: XLookupString() normally returns a single ISO Latin 1 or
+	 * ASCII control character.
 	 */
 
-	if (dispPtr->flags & TK_DISPLAY_XIM_SPOT) {
-	    spot.x = dispPtr->caret.x;
-	    spot.y = dispPtr->caret.y + dispPtr->caret.height;
-	    preedit_attr = XVaCreateNestedList(0, XNSpotLocation, &spot, NULL);
-	    XSetICValues(winPtr->inputContext,
-		    XNPreeditAttributes, preedit_attr, NULL);
-	    XFree(preedit_attr);
-	}
-#endif
-    } else {
+	Tcl_DStringInit(&buf);
+	Tcl_DStringSetLength(&buf, TCL_DSTRING_STATIC_SIZE-1);
 	len = XLookupString(&eventPtr->xkey, Tcl_DStringValue(&buf),
-		Tcl_DStringLength(&buf), NULL, NULL);
+		TCL_DSTRING_STATIC_SIZE, &kePtr->keysym, 0);
+	Tcl_DStringValue(&buf)[len] = '\0';
+
+	if (len == 1) {
+	    len = Tcl_UniCharToUtf((unsigned char) Tcl_DStringValue(&buf)[0],
+		    Tcl_DStringValue(dsPtr));
+	    Tcl_DStringSetLength(dsPtr, len);
+	} else {
+	    /*
+	     * len > 1 should only happen if someone has called XRebindKeysym.
+	     * Assume UTF-8.
+	     */
+
+	    Tcl_DStringSetLength(dsPtr, len);
+	    strncpy(Tcl_DStringValue(dsPtr), Tcl_DStringValue(&buf), len);
+	}
     }
-#else /* TK_USE_INPUT_METHODS */
-    len = XLookupString(&eventPtr->xkey, Tcl_DStringValue(&buf),
-	    Tcl_DStringLength(&buf), NULL, NULL);
-#endif /* TK_USE_INPUT_METHODS */
-    Tcl_DStringSetLength(&buf, len);
 
-    Tcl_ExternalToUtfDString(NULL, Tcl_DStringValue(&buf), len, dsPtr);
-    Tcl_DStringFree(&buf);
+    /*
+     * Cache the string in the event so that if/when we return to this
+     * function, we will be able to produce it without asking X. This stops us
+     * from having to reenter the XIM engine. [Bug 1373712]
+     */
 
+    kePtr->charValuePtr = ckalloc((unsigned) len + 1);
+    kePtr->charValueLen = len;
+    memcpy(kePtr->charValuePtr, Tcl_DStringValue(dsPtr), (unsigned) len + 1);
     return Tcl_DStringValue(dsPtr);
 }
 
 /*
  * When mapping from a keysym to a keycode, need information about the
- * modifier state that should be used so that when they call XKeycodeToKeysym
- * taking into account the xkey.state, they will get back the original keysym.
+ * modifier state to be used so that when they call TkKeycodeToKeysym taking
+ * into account the xkey.state, they will get back the original keysym.
  */
 
 void
@@ -164,27 +236,23 @@ TkpSetKeycodeAndState(
     KeySym keySym,
     XEvent *eventPtr)
 {
-    Display *display;
+    TkDisplay *dispPtr = ((TkWindow *) tkwin)->dispPtr;
     int state;
     KeyCode keycode;
 
-    display = Tk_Display(tkwin);
-
     if (keySym == NoSymbol) {
 	keycode = 0;
     } else {
-	keycode = XKeysymToKeycode(display, keySym);
+	keycode = XKeysymToKeycode(dispPtr->display, keySym);
     }
+    eventPtr->xkey.keycode = keycode;
     if (keycode != 0) {
 	for (state = 0; state < 4; state++) {
-	    if (XKeycodeToKeysym(display, keycode, state) == keySym) {
+	    if (XLookupKeysym(&eventPtr->xkey, state) == keySym) {
 		if (state & 1) {
 		    eventPtr->xkey.state |= ShiftMask;
 		}
 		if (state & 2) {
-		    TkDisplay *dispPtr;
-
-		    dispPtr = ((TkWindow *) tkwin)->dispPtr;
 		    eventPtr->xkey.state |= dispPtr->modeModMask;
 		}
 		break;
@@ -220,15 +288,40 @@ TkpGetKeySym(
 {
     KeySym sym;
     int index;
+    TkKeyEvent* kePtr = (TkKeyEvent*) eventPtr;
 
     /*
-     * Refresh the mapping information if it's stale
+     * Refresh the mapping information if it's stale. This must happen before
+     * we do any input method processing. [Bug 3599312]
      */
 
     if (dispPtr->bindInfoStale) {
 	TkpInitKeymapInfo(dispPtr);
     }
 
+#ifdef TK_USE_INPUT_METHODS
+    /*
+     * If input methods are active, we may already have determined a keysym.
+     * Return it.
+     */
+
+    if (eventPtr->type == KeyPress && dispPtr
+	    && (dispPtr->flags & TK_DISPLAY_USE_IM)) {
+	if (kePtr->charValuePtr == NULL) {
+	    Tcl_DString ds;
+	    TkWindow *winPtr = (TkWindow *)
+		Tk_IdToWindow(eventPtr->xany.display, eventPtr->xany.window);
+
+	    Tcl_DStringInit(&ds);
+	    (void) TkpGetString(winPtr, eventPtr, &ds);
+	    Tcl_DStringFree(&ds);
+	}
+	if (kePtr->charValuePtr != NULL) {
+	    return kePtr->keysym;
+	}
+    }
+#endif
+
     /*
      * Figure out which of the four slots in the keymap vector to use for this
      * key. Refer to Xlib documentation for more info on how this computation
@@ -244,7 +337,8 @@ TkpGetKeySym(
 	    && (eventPtr->xkey.state & LockMask))) {
 	index += 1;
     }
-    sym = XKeycodeToKeysym(dispPtr->display, eventPtr->xkey.keycode, index);
+    sym = TkKeycodeToKeysym(dispPtr, eventPtr->xkey.keycode, 0,
+	    index);
 
     /*
      * Special handling: if the key was shifted because of Lock, but lock is
@@ -258,8 +352,8 @@ TkpGetKeySym(
 		|| ((sym >= XK_Agrave) && (sym <= XK_Odiaeresis))
 		|| ((sym >= XK_Ooblique) && (sym <= XK_Thorn)))) {
 	    index &= ~1;
-	    sym = XKeycodeToKeysym(dispPtr->display, eventPtr->xkey.keycode,
-		    index);
+	    sym = TkKeycodeToKeysym(dispPtr, eventPtr->xkey.keycode,
+		    0, index);
 	}
     }
 
@@ -269,8 +363,8 @@ TkpGetKeySym(
      */
 
     if ((index & 1) && (sym == NoSymbol)) {
-	sym = XKeycodeToKeysym(dispPtr->display, eventPtr->xkey.keycode,
-		index & ~1);
+	sym = TkKeycodeToKeysym(dispPtr, eventPtr->xkey.keycode,
+		0, index & ~1);
     }
     return sym;
 }
@@ -319,7 +413,7 @@ TkpInitKeymapInfo(
 	if (*codePtr == 0) {
 	    continue;
 	}
-	keysym = XKeycodeToKeysym(dispPtr->display, *codePtr, 0);
+	keysym = TkKeycodeToKeysym(dispPtr, *codePtr, 0, 0);
 	if (keysym == XK_Shift_Lock) {
 	    dispPtr->lockUsage = LU_SHIFT;
 	    break;
@@ -345,7 +439,7 @@ TkpInitKeymapInfo(
 	if (*codePtr == 0) {
 	    continue;
 	}
-	keysym = XKeycodeToKeysym(dispPtr->display, *codePtr, 0);
+	keysym = TkKeycodeToKeysym(dispPtr, *codePtr, 0, 0);
 	if (keysym == XK_Mode_switch) {
 	    dispPtr->modeModMask |= ShiftMask << (i/modMapPtr->max_keypermod);
 	}
