@@ -10,8 +10,6 @@
  *
  * See the file "license.terms" for information on usage and redistribution of
  * this file, and for a DISCLAIMER OF ALL WARRANTIES.
- *
- * RCS: @(#) $Id: tkUnixSend.c,v 1.20.2.1 2009/08/25 08:48:16 dkf Exp $
  */
 
 #include "tkUnixInt.h"
@@ -679,54 +677,74 @@ ServerSecure(
     int numHosts, secure;
     Bool enabled;
 
-    secure = 0;
     addrPtr = XListHosts(dispPtr->display, &numHosts, &enabled);
-    if (enabled) {
-	if (numHosts == 0) {
-	    secure = 1;
-	}
-
+    if (!enabled) {
+    insecure:
+	secure = 0;
+    } else if (numHosts == 0) {
+	secure = 1;
+    } else {
 	/*
 	 * Recent versions of X11 have the extra feature of allowing more
 	 * sophisticated authorization checks to be performed than the dozy
 	 * old ones that used to plague xhost usage. However, not all deployed
 	 * versions of Xlib know how to deal with this feature, so this code
 	 * is conditional on having the right #def in place. [Bug 1909931]
+	 *
+	 * Note that at this point we know that there's at least one entry in
+	 * the list returned by XListHosts. However there may be multiple
+	 * entries; as long as each is one of either 'SI:localhost:*' or
+	 * 'SI:localgroup:*' then we will claim to be secure enough.
 	 */
 
 #ifdef FamilyServerInterpreted
-	if (numHosts == 1 && addrPtr[0].family == FamilyServerInterpreted) {
-	    XServerInterpretedAddress *siPtr =
-		    (XServerInterpretedAddress *) addrPtr[0].address;
+	XServerInterpretedAddress *siPtr;
+	int i;
 
-	    if (siPtr->typelength==9 && !memcmp(siPtr->type,"localuser",9)) {
+	for (i=0 ; i<numHosts ; i++) {
+	    if (addrPtr[i].family != FamilyServerInterpreted) {
 		/*
-		 * We don't check the username here. This is because it's
-		 * officially non-portable and we are just making sure there
-		 * aren't silly misconfigurations. (Apparently 'root' is not a
-		 * very good choice, but we still don't put any effort in to
-		 * spot that.)
+		 * We don't understand what the X server is letting in, so we
+		 * err on the side of safety.
 		 */
 
-		secure = 1;
-	    } else if (siPtr->typelength == 10
-		    && !memcmp(siPtr->type, "localgroup", 10)) {
-		/*
-		 * Similarly to above, we don't attempt to peek inside server
-		 * interpreted group names. If someone set it, it's what they
-		 * want and we assume it's OK.
-		 */
-
-		secure = 1;
+		goto insecure;
 	    }
+	    siPtr = (XServerInterpretedAddress *) addrPtr[0].address;
 
 	    /*
-	     * The other defined types of server-interpreted controls involve
-	     * particular hosts; these are still insecure for the same reasons
-	     * that classic xhost access is insecure.
+	     * We don't check the username or group here. This is because it's
+	     * officially non-portable and we are just making sure there
+	     * aren't silly misconfigurations. (Apparently 'root' is not a
+	     * very good choice, but we still don't put any effort in to spot
+	     * that.) However we do check to see that the constraints are
+	     * imposed against the connecting user and/or group.
 	     */
+
+	    if (       !(siPtr->typelength == 9 /* ==strlen("localuser") */
+			&& !memcmp(siPtr->type, "localuser", 9))
+		    && !(siPtr->typelength == 10 /* ==strlen("localgroup") */
+			&& !memcmp(siPtr->type, "localgroup", 10))) {
+		/*
+		 * The other defined types of server-interpreted controls
+		 * involve particular hosts. These are still insecure for the
+		 * same reasons that classic xhost access is insecure; there's
+		 * just no way to be sure that the users on those systems are
+		 * the ones who should be allowed to connect to this display.
+		 */
+
+		goto insecure;
+	    }
 	}
-#endif
+	secure = 1;
+#else
+	/*
+	 * We don't understand what the X server is letting in, so we err on
+	 * the side of safety.
+	 */
+
+	goto insecure;
+#endif /* FamilyServerInterpreted */
     }
     if (addrPtr != NULL) {
 	XFree((char *) addrPtr);
@@ -1004,7 +1022,7 @@ Tk_SendCmd(
 	localInterp = riPtr->interp;
 	Tcl_Preserve((ClientData) localInterp);
 	if (firstArg == (argc-1)) {
-	    result = Tcl_GlobalEval(localInterp, argv[firstArg]);
+	    result = Tcl_EvalEx(localInterp, argv[firstArg], -1, TCL_EVAL_GLOBAL);
 	} else {
 	    Tcl_DStringInit(&request);
 	    Tcl_DStringAppend(&request, argv[firstArg], -1);
@@ -1012,7 +1030,7 @@ Tk_SendCmd(
 		Tcl_DStringAppend(&request, " ", 1);
 		Tcl_DStringAppend(&request, argv[i], -1);
 	    }
-	    result = Tcl_GlobalEval(localInterp, Tcl_DStringValue(&request));
+	    result = Tcl_EvalEx(localInterp, Tcl_DStringValue(&request), -1, TCL_EVAL_GLOBAL);
 	    Tcl_DStringFree(&request);
 	}
 	if (interp != localInterp) {
@@ -1332,12 +1350,11 @@ SendInit(
      * for it.
      */
 
-    dispPtr->commTkwin = Tk_CreateWindow(interp, (Tk_Window) NULL,
-	    "_comm", DisplayString(dispPtr->display));
-    if (dispPtr->commTkwin == NULL) {
-	Tcl_Panic("Tk_CreateWindow failed in SendInit!");
-    }
+    dispPtr->commTkwin = (Tk_Window) TkAllocWindow(dispPtr,
+    	DefaultScreen(dispPtr->display), NULL);
     Tcl_Preserve((ClientData) dispPtr->commTkwin);
+    ((TkWindow *) dispPtr->commTkwin)->flags |=TK_TOP_HIERARCHY|TK_TOP_LEVEL|TK_HAS_WRAPPER|TK_WIN_MANAGED;
+    TkWmNewWindow((TkWindow *) dispPtr->commTkwin);
     atts.override_redirect = True;
     Tk_ChangeWindowAttributes(dispPtr->commTkwin,
 	    CWOverrideRedirect, &atts);
@@ -1539,7 +1556,7 @@ SendEventProc(
 	    remoteInterp = riPtr->interp;
 	    Tcl_Preserve((ClientData) remoteInterp);
 
-	    result = Tcl_GlobalEval(remoteInterp, script);
+	    result = Tcl_EvalEx(remoteInterp, script, -1, TCL_EVAL_GLOBAL);
 
 	    /*
 	     * The call to Tcl_Release may have released the interpreter which
