@@ -586,13 +586,23 @@ TkpScanWindowId(
     CONST char *string,
     Window *idPtr)
 {
-    int value;
+    int code;
+    Tcl_Obj obj;
 
-    if (Tcl_GetInt(interp, string, &value) != TCL_OK) {
-	return TCL_ERROR;
+    obj.refCount = 1;
+    obj.bytes = (char *) string;	/* DANGER?! */
+    obj.length = strlen(string);
+    obj.typePtr = NULL;
+
+    code = Tcl_GetLongFromObj(interp, &obj, (long *)idPtr);
+
+    if (obj.refCount > 1) {
+	Tcl_Panic("invalid sharing of Tcl_Obj on C stack");
     }
-    *idPtr = (Window) value;
-    return TCL_OK;
+    if (obj.typePtr && obj.typePtr->freeIntRepProc) {
+	obj.typePtr->freeIntRepProc(&obj);
+    }
+    return code;
 }
 
 /*
