@@ -200,7 +200,7 @@ TkpGetColor(
 
     if (((strncasecmp(name, "system", 6) == 0)
 	    && FindSystemColor(name+6, &color, &index))
-	    || XParseColor(Tk_Display(tkwin), Tk_Colormap(tkwin), name,
+	    || TkParseColor(Tk_Display(tkwin), Tk_Colormap(tkwin), name,
 		    &color)) {
 	winColPtr = (WinColor *) ckalloc(sizeof(WinColor));
 	winColPtr->info.color = color;
@@ -386,13 +386,13 @@ XAllocColor(
 
 	color->pixel = PALETTERGB(entry.peRed, entry.peGreen, entry.peBlue);
 	entryPtr = Tcl_CreateHashEntry(&cmap->refCounts,
-		(char *) color->pixel, &new);
+		INT2PTR(color->pixel), &new);
 	if (new) {
 	    refCount = 1;
 	} else {
-	    refCount = ((int) Tcl_GetHashValue(entryPtr)) + 1;
+	    refCount = (PTR2INT(Tcl_GetHashValue(entryPtr))) + 1;
 	}
-	Tcl_SetHashValue(entryPtr, (ClientData)refCount);
+	Tcl_SetHashValue(entryPtr, INT2PTR(refCount));
     } else {
 	/*
 	 * Determine what color will actually be used on non-colormap systems.
@@ -426,7 +426,7 @@ XAllocColor(
  *----------------------------------------------------------------------
  */
 
-void
+int
 XFreeColors(
     Display *display,
     Colormap colormap,
@@ -452,11 +452,11 @@ XFreeColors(
 	 */
 
 	for (i = 0; i < npixels; i++) {
-	    entryPtr = Tcl_FindHashEntry(&cmap->refCounts, (char *) pixels[i]);
+	    entryPtr = Tcl_FindHashEntry(&cmap->refCounts, INT2PTR(pixels[i]));
 	    if (!entryPtr) {
 		Tcl_Panic("Tried to free a color that isn't allocated.");
 	    }
-	    refCount = (int) Tcl_GetHashValue(entryPtr) - 1;
+	    refCount = PTR2INT(Tcl_GetHashValue(entryPtr)) - 1;
 	    if (refCount == 0) {
 		cref = pixels[i] & 0x00ffffff;
 		index = GetNearestPaletteIndex(cmap->palette, cref);
@@ -474,11 +474,12 @@ XFreeColors(
 		}
 		Tcl_DeleteHashEntry(entryPtr);
 	    } else {
-		Tcl_SetHashValue(entryPtr, (ClientData)refCount);
+		Tcl_SetHashValue(entryPtr, INT2PTR(refCount));
 	    }
 	}
     }
     ReleaseDC(NULL, dc);
+    return Success;
 }
 
 /*
@@ -535,9 +536,9 @@ XCreateColormap(
     Tcl_InitHashTable(&cmap->refCounts, TCL_ONE_WORD_KEYS);
     for (i = 0; i < logPalettePtr->palNumEntries; i++) {
 	entryPtr = logPalettePtr->palPalEntry + i;
-	hashPtr = Tcl_CreateHashEntry(&cmap->refCounts, (char*) PALETTERGB(
-		entryPtr->peRed, entryPtr->peGreen, entryPtr->peBlue), &new);
-	Tcl_SetHashValue(hashPtr, (ClientData)1);
+	hashPtr = Tcl_CreateHashEntry(&cmap->refCounts, INT2PTR(PALETTERGB(
+		entryPtr->peRed, entryPtr->peGreen, entryPtr->peBlue)), &new);
+	Tcl_SetHashValue(hashPtr, INT2PTR(1));
     }
 
     return (Colormap)cmap;
@@ -560,7 +561,7 @@ XCreateColormap(
  *----------------------------------------------------------------------
  */
 
-void
+int
 XFreeColormap(
     Display *display,
     Colormap colormap)
@@ -572,6 +573,7 @@ XFreeColormap(
     }
     Tcl_DeleteHashTable(&cmap->refCounts);
     ckfree((char *) cmap);
+    return Success;
 }
 
 /*
