@@ -1,17 +1,15 @@
-/* 
+/*
  * tkWinConfig.c --
  *
- *	This module implements the Windows system defaults for
- *	the configuration package.
+ *	This module implements the Windows system defaults for the
+ *	configuration package.
  *
  * Copyright (c) 1997 by Sun Microsystems, Inc.
  *
- * See the file "license.terms" for information on usage and redistribution
- * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
+ * See the file "license.terms" for information on usage and redistribution of
+ * this file, and for a DISCLAIMER OF ALL WARRANTIES.
  */
 
-#include "tk.h"
-#include "tkInt.h"
 #include "tkWinInt.h"
 
 
@@ -20,13 +18,13 @@
  *
  * TkpGetSystemDefault --
  *
- *	Given a dbName and className for a configuration option,
- *	return a string representation of the option.
+ *	Given a dbName and className for a configuration option, return a
+ *	string representation of the option.
  *
  * Results:
- *	Returns a Tk_Uid that is the string identifier that identifies
- *	this option. Returns NULL if there are no system defaults
- *	that match this pair.
+ *	Returns a Tk_Uid that is the string identifier that identifies this
+ *	option. Returns NULL if there are no system defaults that match this
+ *	pair.
  *
  * Side effects:
  *	None, once the package is initialized.
@@ -38,7 +36,7 @@ Tcl_Obj *
 TkpGetSystemDefault(
     Tk_Window tkwin,		/* A window to use. */
     CONST char *dbName,		/* The option database name. */
-    CONST char *className)		/* The name of the option class. */
+    CONST char *className)	/* The name of the option class. */
 {
     Tcl_Obj *valueObjPtr;
     Tk_Uid classUid;
@@ -56,3 +54,11 @@ TkpGetSystemDefault(
 
     return valueObjPtr;
 }
+
+/*
+ * Local Variables:
+ * mode: c
+ * c-basic-offset: 4
+ * fill-column: 78
+ * End:
+ */
