@@ -24,7 +24,7 @@
  * Translation table between X gc functions and Win32 raster op modes.
  */
 
-int tkpWinRopModes[] = {
+CONST int tkpWinRopModes[] = {
     R2_BLACK,			/* GXclear */
     R2_MASKPEN,			/* GXand */
     R2_MASKPENNOT,		/* GXandReverse */
@@ -55,7 +55,7 @@ int tkpWinRopModes[] = {
 #define SRCORREVERSE	(DWORD)0x00DD0228 /* dest = source OR (NOT dest) */
 #define SRCNAND		(DWORD)0x007700E6 /* dest = NOT (source AND dest) */
 
-int tkpWinBltModes[] = {
+CONST int tkpWinBltModes[] = {
     BLACKNESS,			/* GXclear */
     SRCAND,			/* GXand */
     SRCERASE,			/* GXandReverse */
@@ -93,9 +93,10 @@ int tkpWinBltModes[] = {
 /*
  * Macros used later in the file.
  */
-
-#define MIN(a,b)	((a>b) ? b : a)
-#define MAX(a,b)	((a<b) ? b : a)
+#ifndef MIN
+#   define MIN(a,b)	((a>b) ? b : a)
+#   define MAX(a,b)	((a<b) ? b : a)
+#endif
 
 /*
  * The followng typedef is used to pass Windows GDI drawing functions.
@@ -115,7 +116,7 @@ static Tcl_ThreadDataKey dataKey;
 
 static POINT *		ConvertPoints(XPoint *points, int npoints, int mode,
 			    RECT *bbox);
-static void		DrawOrFillArc(Display *display, Drawable d, GC gc,
+static int		DrawOrFillArc(Display *display, Drawable d, GC gc,
 			    int x, int y, unsigned int width,
 			    unsigned int height, int start, int extent,
 			    int fill);
@@ -296,7 +297,7 @@ ConvertPoints(
  *----------------------------------------------------------------------
  */
 
-void
+int
 XCopyArea(
     Display *display,
     Drawable src,
@@ -332,6 +333,7 @@ XCopyArea(
 	TkWinReleaseDrawableDC(dest, destDC, &destState);
     }
     TkWinReleaseDrawableDC(src, srcDC, &srcState);
+    return Success;
 }
 
 /*
@@ -352,7 +354,7 @@ XCopyArea(
  *----------------------------------------------------------------------
  */
 
-void
+int
 XCopyPlane(
     Display *display,
     Drawable src,
@@ -479,6 +481,7 @@ XCopyPlane(
 	TkWinReleaseDrawableDC(dest, destDC, &destState);
     }
     TkWinReleaseDrawableDC(src, srcDC, &srcState);
+    return Success;
 }
 
 /*
@@ -498,7 +501,7 @@ XCopyPlane(
  *----------------------------------------------------------------------
  */
 
-void
+int
 TkPutImage(
     unsigned long *colors,	/* Array of pixel values used by this image.
 				 * May be NULL. */
@@ -587,7 +590,7 @@ TkPutImage(
 	Tcl_Panic("Fail to allocate bitmap\n");
 	DeleteDC(dcMem);
     	TkWinReleaseDrawableDC(d, dc, &state);
-	return;
+	return BadValue;
     }
     bitmap = SelectObject(dcMem, bitmap);
     BitBlt(dc, dest_x, dest_y, (int) width, (int) height, dcMem, src_x, src_y,
@@ -595,6 +598,7 @@ TkPutImage(
     DeleteObject(SelectObject(dcMem, bitmap));
     DeleteDC(dcMem);
     TkWinReleaseDrawableDC(d, dc, &state);
+    return Success;
 }
 
 /*
@@ -613,7 +617,7 @@ TkPutImage(
  *----------------------------------------------------------------------
  */
 
-void
+int
 XFillRectangles(
     Display *display,
     Drawable d,
@@ -628,7 +632,7 @@ XFillRectangles(
     HBRUSH brush, oldBrush;
 
     if (d == None) {
-	return;
+	return BadDrawable;
     }
 
     dc = TkWinGetDrawableDC(display, d, &state);
@@ -715,6 +719,7 @@ XFillRectangles(
     }
     DeleteObject(brush);
     TkWinReleaseDrawableDC(d, dc, &state);
+    return Success;
 }
 
 /*
@@ -744,7 +749,7 @@ RenderObject(
     HPEN pen,
     WinDrawFunc func)
 {
-    RECT rect;
+    RECT rect = {0, 0, 0, 0};
     HPEN oldPen;
     HBRUSH oldBrush;
     POINT *winPoints = ConvertPoints(points, npoints, mode, &rect);
@@ -864,7 +869,7 @@ RenderObject(
  *----------------------------------------------------------------------
  */
 
-void
+int
 XDrawLines(
     Display *display,
     Drawable d,
@@ -878,7 +883,7 @@ XDrawLines(
     HDC dc;
 
     if (d == None) {
-	return;
+	return BadDrawable;
     }
 
     dc = TkWinGetDrawableDC(display, d, &state);
@@ -889,6 +894,7 @@ XDrawLines(
     DeleteObject(pen);
 
     TkWinReleaseDrawableDC(d, dc, &state);
+    return Success;
 }
 
 /*
@@ -907,7 +913,7 @@ XDrawLines(
  *----------------------------------------------------------------------
  */
 
-void
+int
 XFillPolygon(
     Display *display,
     Drawable d,
@@ -922,7 +928,7 @@ XFillPolygon(
     HDC dc;
 
     if (d == None) {
-	return;
+	return BadDrawable;
     }
 
     dc = TkWinGetDrawableDC(display, d, &state);
@@ -931,6 +937,7 @@ XFillPolygon(
     RenderObject(dc, gc, points, npoints, mode, pen, Polygon);
 
     TkWinReleaseDrawableDC(d, dc, &state);
+    return Success;
 }
 
 /*
@@ -949,7 +956,7 @@ XFillPolygon(
  *----------------------------------------------------------------------
  */
 
-void
+int
 XDrawRectangle(
     Display *display,
     Drawable d,
@@ -963,7 +970,7 @@ XDrawRectangle(
     HDC dc;
 
     if (d == None) {
-	return;
+	return BadDrawable;
     }
 
     dc = TkWinGetDrawableDC(display, d, &state);
@@ -979,6 +986,7 @@ XDrawRectangle(
     DeleteObject(SelectObject(dc, oldPen));
     SelectObject(dc, oldBrush);
     TkWinReleaseDrawableDC(d, dc, &state);
+    return Success;
 }
 
 /*
@@ -997,7 +1005,7 @@ XDrawRectangle(
  *----------------------------------------------------------------------
  */
 
-void
+int
 XDrawArc(
     Display *display,
     Drawable d,
@@ -1008,7 +1016,7 @@ XDrawArc(
 {
     display->request++;
 
-    DrawOrFillArc(display, d, gc, x, y, width, height, start, extent, 0);
+    return DrawOrFillArc(display, d, gc, x, y, width, height, start, extent, 0);
 }
 
 /*
@@ -1027,7 +1035,7 @@ XDrawArc(
  *----------------------------------------------------------------------
  */
 
-void
+int
 XFillArc(
     Display *display,
     Drawable d,
@@ -1038,7 +1046,7 @@ XFillArc(
 {
     display->request++;
 
-    DrawOrFillArc(display, d, gc, x, y, width, height, start, extent, 1);
+    return DrawOrFillArc(display, d, gc, x, y, width, height, start, extent, 1);
 }
 
 /*
@@ -1058,7 +1066,7 @@ XFillArc(
  *----------------------------------------------------------------------
  */
 
-static void
+static int
 DrawOrFillArc(
     Display *display,
     Drawable d,
@@ -1078,7 +1086,7 @@ DrawOrFillArc(
     double radian_start, radian_end, xr, yr;
 
     if (d == None) {
-	return;
+	return BadDrawable;
     }
 
     dc = TkWinGetDrawableDC(display, d, &state);
@@ -1151,6 +1159,7 @@ DrawOrFillArc(
     }
     DeleteObject(SelectObject(dc, oldPen));
     TkWinReleaseDrawableDC(d, dc, &state);
+    return Success;
 }
 
 /*
