@@ -8,8 +8,6 @@
  *
  * See the file "license.terms" for information on usage and redistribution of
  * this file, and for a DISCLAIMER OF ALL WARRANTIES.
- *
- * RCS: @(#) $Id: winMain.c,v 1.26 2007/12/13 15:28:56 dgp Exp $
  */
 
 #include "tkInt.h"
@@ -18,20 +16,29 @@
 #undef WIN32_LEAN_AND_MEAN
 #include <locale.h>
 
+#if defined(__GNUC__)
+int _CRT_glob = 0;
+#endif /* __GNUC__ */
 
 /*
  * The following declarations refer to internal Tk routines. These interfaces
  * are available for use, but are not supported.
  */
+#ifdef TK_TEST
+extern Tcl_PackageInitProc Tktest_Init;
+#endif /* TK_TEST */
+
+#if defined(STATIC_BUILD) && TCL_USE_STATIC_PACKAGES
+extern Tcl_PackageInitProc Registry_Init;
+extern Tcl_PackageInitProc Dde_Init;
+extern Tcl_PackageInitProc Dde_SafeInit;
+#endif
 
 /*
  * Forward declarations for procedures defined later in this file:
  */
 
 static void		WishPanic(CONST char *format, ...);
-#ifdef TK_TEST
-extern int		Tktest_Init(Tcl_Interp *interp);
-#endif /* TK_TEST */
 
 static BOOL consoleRequired = TRUE;
 
@@ -148,6 +155,9 @@ int
 Tcl_AppInit(
     Tcl_Interp *interp)		/* Interpreter for application. */
 {
+#define TK_MAX_WARN_LEN 1024
+    WCHAR msgString[TK_MAX_WARN_LEN + 5];
+
     if (Tcl_Init(interp) == TCL_ERROR) {
 	goto error;
     }
@@ -167,20 +177,15 @@ Tcl_AppInit(
 	}
     }
 #if defined(STATIC_BUILD) && TCL_USE_STATIC_PACKAGES
-    {
-	extern Tcl_PackageInitProc Registry_Init;
-	extern Tcl_PackageInitProc Dde_Init;
+    if (Registry_Init(interp) == TCL_ERROR) {
+	goto error;
+    }
+    Tcl_StaticPackage(interp, "registry", Registry_Init, NULL);
 
-	if (Registry_Init(interp) == TCL_ERROR) {
-	    return TCL_ERROR;
-	}
-	Tcl_StaticPackage(interp, "registry", Registry_Init, NULL);
-
-	if (Dde_Init(interp) == TCL_ERROR) {
-	    return TCL_ERROR;
-	}
-	Tcl_StaticPackage(interp, "dde", Dde_Init, NULL);
-   }
+    if (Dde_Init(interp) == TCL_ERROR) {
+	goto error;
+    }
+    Tcl_StaticPackage(interp, "dde", Dde_Init, NULL);
 #endif
 
 #ifdef TK_TEST
@@ -190,12 +195,43 @@ Tcl_AppInit(
     Tcl_StaticPackage(interp, "Tktest", Tktest_Init, NULL);
 #endif /* TK_TEST */
 
+    /*
+     * Call the init procedures for included packages. Each call should look
+     * like this:
+     *
+     * if (Mod_Init(interp) == TCL_ERROR) {
+     *     return TCL_ERROR;
+     * }
+     *
+     * where "Mod" is the name of the module. (Dynamically-loadable packages
+     * should have the same entry-point name.)
+     */
+
+    /*
+     * Call Tcl_CreateCommand for application-specific commands, if they
+     * weren't already created by the init procedures called above.
+     */
+
+    /*
+     * Specify a user-specific startup file to invoke if the application is
+     * run interactively. Typically the startup file is "~/.apprc" where "app"
+     * is the name of the application. If this line is deleted then no user-
+     * specific startup file will be run under any conditions.
+     */
+
     Tcl_SetVar(interp, "tcl_rcFileName", "~/wishrc.tcl", TCL_GLOBAL_ONLY);
     return TCL_OK;
 
 error:
+    MultiByteToWideChar(CP_UTF8, 0, Tcl_GetStringResult(interp), -1,
+	    msgString, TK_MAX_WARN_LEN);
+    /*
+     * Truncate MessageBox string if it is too long to not overflow the screen
+     * and cause possible oversized window error.
+     */
+	memcpy(msgString + TK_MAX_WARN_LEN, L" ...", 5 * sizeof(WCHAR));
     MessageBeep(MB_ICONEXCLAMATION);
-    MessageBox(NULL, Tcl_GetStringResult(interp), "Error in Wish",
+    MessageBoxW(NULL, msgString, L"Error in Wish",
 	    MB_ICONSTOP | MB_OK | MB_TASKMODAL | MB_SETFOREGROUND);
     ExitProcess(1);
 
@@ -227,13 +263,20 @@ WishPanic(
     CONST char *format, ...)
 {
     va_list argList;
-    char buf[1024];
+    char buf[TK_MAX_WARN_LEN];
+    WCHAR msgString[TK_MAX_WARN_LEN + 5];
 
     va_start(argList, format);
     vsprintf(buf, format, argList);
 
+    MultiByteToWideChar(CP_UTF8, 0, buf, -1, msgString, TK_MAX_WARN_LEN);
+    /*
+     * Truncate MessageBox string if it is too long to not overflow the screen
+     * and cause possible oversized window error.
+     */
+	memcpy(msgString + TK_MAX_WARN_LEN, L" ...", 5 * sizeof(WCHAR));
     MessageBeep(MB_ICONEXCLAMATION);
-    MessageBox(NULL, buf, "Fatal Error in Wish",
+    MessageBoxW(NULL, msgString, L"Fatal Error in Wish",
 	    MB_ICONSTOP | MB_OK | MB_TASKMODAL | MB_SETFOREGROUND);
 #ifdef _MSC_VER
     DebugBreak();
@@ -241,7 +284,7 @@ WishPanic(
     ExitProcess(1);
 }
 
-#if !defined(__GNUC__) || defined(TK_TEST)
+#if defined(TK_TEST)
 /*
  *----------------------------------------------------------------------
  *
@@ -283,8 +326,7 @@ main(
     Tk_Main(argc, argv, Tcl_AppInit);
     return 0;
 }
-#endif /* !__GNUC__ || TK_TEST */
-
+#endif /* TK_TEST */
 /*
  * Local Variables:
  * mode: c
